@@ -2,7 +2,7 @@
 mod support;
 
 macro_rules! impl_mat4_tests {
-    ($t:ident, $const_new:ident, $newmat4:ident, $newvec4:ident, $newvec3:ident, $mat4:ident, $quat:ident, $vec4:ident, $vec3:ident) => {
+    ($t:ident, $const_new:ident, $newmat4:ident, $newvec4:ident, $newvec3:ident, $mat4:ident, $mat3:ident, $quat:ident, $vec4:ident, $vec3:ident) => {
         use core::$t::INFINITY;
         use core::$t::NAN;
         use core::$t::NEG_INFINITY;
@@ -107,6 +107,22 @@ macro_rules! impl_mat4_tests {
             assert_eq!($newvec4(2.0, 6.0, 10.0, 14.0), m.row(1));
             assert_eq!($newvec4(3.0, 7.0, 11.0, 15.0), m.row(2));
             assert_eq!($newvec4(4.0, 8.0, 12.0, 16.0), m.row(3));
+
+            *m.col_mut(0) = m.col(0).wzyx();
+            *m.col_mut(1) = m.col(1).wzyx();
+            *m.col_mut(2) = m.col(2).wzyx();
+            *m.col_mut(3) = m.col(3).wzyx();
+            assert_eq!($newvec4(4.0, 3.0, 2.0, 1.0), m.col(0));
+            assert_eq!($newvec4(8.0, 7.0, 6.0, 5.0), m.col(1));
+            assert_eq!($newvec4(12.0, 11.0, 10.0, 9.0), m.col(2));
+            assert_eq!($newvec4(16.0, 15.0, 14.0, 13.0), m.col(3));
+
+            should_panic!({ $mat4::ZERO.col(4) });
+            should_panic!({
+                let mut m = $mat4::ZERO;
+                m.col_mut(4);
+            });
+            should_panic!({ $mat4::ZERO.row(4) });
         }
 
         #[test]
@@ -162,6 +178,27 @@ macro_rules! impl_mat4_tests {
             let rot_z1 = $mat4::from_rotation_z(deg(180.0));
             let rot_z2 = $mat4::from_axis_angle($vec3::Z, deg(180.0));
             assert_approx_eq!(rot_z1, rot_z2);
+
+            assert_approx_eq!($mat4::IDENTITY, $mat4::from_quat($quat::IDENTITY));
+
+            should_glam_assert!({ $mat4::from_axis_angle($vec3::ZERO, 0.0) });
+            should_glam_assert!({ $mat4::from_quat($quat::from_xyzw(0.0, 0.0, 0.0, 0.0)) });
+        }
+
+        #[test]
+        fn test_from_mat3() {
+            let m3 =
+                $mat3::from_cols_array_2d(&[[1.0, 2.0, 3.0], [4.0, 5.0, 6.0], [7.0, 8.0, 9.0]]);
+            let m4 = $mat4::from_mat3(m3);
+            assert_eq!(
+                $mat4::from_cols_array_2d(&[
+                    [1.0, 2.0, 3.0, 0.0],
+                    [4.0, 5.0, 6.0, 0.0],
+                    [7.0, 8.0, 9.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0]
+                ]),
+                m4
+            );
         }
 
         #[test]
@@ -197,6 +234,9 @@ macro_rules! impl_mat4_tests {
                 $newvec3(4.0, 2.0, 1.0),
                 m.project_point3($newvec3(2.0, 2.0, 2.0))
             );
+
+            should_glam_assert!({ $mat4::ZERO.transform_vector3($vec3::X) });
+            should_glam_assert!({ $mat4::ZERO.transform_point3($vec3::X) });
         }
 
         #[test]
@@ -238,6 +278,8 @@ macro_rules! impl_mat4_tests {
                 m.transform_point3($vec3::new(1.0, 1.0, 1.0)),
                 $vec3::new(2.0, 4.0, 8.0)
             );
+
+            should_glam_assert!({ $mat4::from_scale($vec3::ZERO) });
         }
 
         #[test]
@@ -314,6 +356,8 @@ macro_rules! impl_mat4_tests {
             let m_inv = m.inverse();
             assert_approx_eq!($mat4::IDENTITY, m * m_inv, 1.0e-5);
             assert_approx_eq!($mat4::IDENTITY, m_inv * m, 1.0e-5);
+
+            should_glam_assert!({ $mat4::ZERO.inverse() });
         }
 
         #[test]
@@ -391,6 +435,20 @@ macro_rules! impl_mat4_tests {
                 $mat4::from_scale_rotation_translation(out_scale, out_rotation, out_translation),
                 1e-6
             );
+
+            should_glam_assert!({
+                $mat4::from_scale_rotation_translation(
+                    $vec3::ONE,
+                    $quat::from_xyzw(0.0, 0.0, 0.0, 0.0),
+                    $vec3::ZERO,
+                )
+            });
+            should_glam_assert!({
+                $mat4::from_rotation_translation($quat::from_xyzw(0.0, 0.0, 0.0, 0.0), $vec3::ZERO)
+            });
+            // TODO: should check scale
+            // should_glam_assert!({ $mat4::from_scale_rotation_translation($vec3::ZERO, $quat::IDENTITY, $vec3::ZERO) });
+            should_glam_assert!({ $mat4::ZERO.to_scale_rotation_translation() });
         }
 
         #[test]
@@ -403,6 +461,9 @@ macro_rules! impl_mat4_tests {
             let point = $vec3::new(1.0, 0.0, 0.0);
             assert_approx_eq!(lh.transform_point3(point), $vec3::new(0.0, 1.0, 5.0));
             assert_approx_eq!(rh.transform_point3(point), $vec3::new(0.0, 1.0, -5.0));
+
+            should_glam_assert!({ $mat4::look_at_lh($vec3::ONE, $vec3::ZERO, $vec3::ZERO) });
+            should_glam_assert!({ $mat4::look_at_rh($vec3::ONE, $vec3::ZERO, $vec3::ZERO) });
         }
 
         #[test]
@@ -429,6 +490,9 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, 0.0, 5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_lh(0.0, 1.0, 1.0, 0.0) });
+            should_glam_assert!({ $mat4::perspective_lh(0.0, 1.0, 0.0, 1.0) });
         }
 
         #[test]
@@ -442,6 +506,8 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, 0.0, 5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_infinite_lh(0.0, 1.0, 0.0) });
         }
 
         #[test]
@@ -455,6 +521,8 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, 5.0, 5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_infinite_reverse_lh(0.0, 1.0, 0.0) });
         }
 
         #[test]
@@ -468,6 +536,9 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, -15.0, -5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_rh(0.0, 1.0, 1.0, 0.0) });
+            should_glam_assert!({ $mat4::perspective_rh(0.0, 1.0, 0.0, 1.0) });
         }
 
         #[test]
@@ -481,6 +552,8 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, -10.0, -5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_infinite_rh(0.0, 1.0, 0.0) });
         }
 
         #[test]
@@ -494,6 +567,8 @@ macro_rules! impl_mat4_tests {
             let original = $vec3::new(5.0, 5.0, 5.0);
             let projected = projection * original.extend(1.0);
             assert_approx_eq!($vec4::new(2.5, 5.0, 5.0, -5.0), projected);
+
+            should_glam_assert!({ $mat4::perspective_infinite_reverse_rh(0.0, 1.0, 0.0) });
         }
 
         #[test]
@@ -543,6 +618,22 @@ macro_rules! impl_mat4_tests {
             assert_eq!($mat4::ZERO, m0 - m0);
             assert_approx_eq!(m0, m0 * $mat4::IDENTITY);
             assert_approx_eq!(m0, $mat4::IDENTITY * m0);
+
+            let mut m1 = m0;
+            m1 *= 2.0;
+            assert_eq!(m0x2, m1);
+
+            let mut m1 = m0;
+            m1 += m0;
+            assert_eq!(m0x2, m1);
+
+            let mut m1 = m0;
+            m1 -= m0;
+            assert_eq!($mat4::ZERO, m1);
+
+            let mut m1 = $mat4::IDENTITY;
+            m1 *= m0;
+            assert_approx_eq!(m0, m1);
         }
 
         #[test]
@@ -554,6 +645,22 @@ macro_rules! impl_mat4_tests {
             );
         }
 
+        #[test]
+        fn test_mat4_to_from_slice() {
+            const MATRIX1D: [$t; 16] = [
+                1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0, 14.0, 15.0,
+                16.0,
+            ];
+            let m = $mat4::from_cols_slice(&MATRIX1D);
+            assert_eq!($mat4::from_cols_array(&MATRIX1D), m);
+            let mut out: [$t; 16] = Default::default();
+            m.write_cols_to_slice(&mut out);
+            assert_eq!(MATRIX1D, out);
+
+            should_panic!({ $mat4::from_cols_slice(&[0.0; 15]) });
+            should_panic!({ $mat4::IDENTITY.write_cols_to_slice(&mut [0.0; 15]) });
+        }
+
         #[cfg(feature = "std")]
         #[test]
         fn test_sum() {
@@ -580,7 +687,7 @@ macro_rules! impl_mat4_tests {
 
 mod mat4 {
     use super::support::deg;
-    use glam::{const_mat4, mat4, vec3, vec4, Mat4, Quat, Vec3, Vec4};
+    use glam::{const_mat4, mat4, swizzles::*, vec3, vec4, Mat3, Mat4, Quat, Vec3, Vec4};
 
     #[test]
     fn test_align() {
@@ -628,12 +735,12 @@ mod mat4 {
         );
     }
 
-    impl_mat4_tests!(f32, const_mat4, mat4, vec4, vec3, Mat4, Quat, Vec4, Vec3);
+    impl_mat4_tests!(f32, const_mat4, mat4, vec4, vec3, Mat4, Mat3, Quat, Vec4, Vec3);
 }
 
 mod dmat4 {
     use super::support::deg;
-    use glam::{const_dmat4, dmat4, dvec3, dvec4, DMat4, DQuat, DVec3, DVec4};
+    use glam::{const_dmat4, dmat4, dvec3, dvec4, swizzles::*, DMat3, DMat4, DQuat, DVec3, DVec4};
 
     #[test]
     fn test_align() {
@@ -649,6 +756,7 @@ mod dmat4 {
         dvec4,
         dvec3,
         DMat4,
+        DMat3,
         DQuat,
         DVec4,
         DVec3
