@@ -135,6 +135,8 @@ macro_rules! impl_vec2_tests {
             assert_eq!(vec(6, 6).clamp(min, max), vec(6, 6));
             assert_eq!(vec(7, 7).clamp(min, max), vec(6, 7));
             assert_eq!(vec(9, 9).clamp(min, max), vec(6, 8));
+
+            should_glam_assert!({ $vec2::clamp($vec2::ZERO, $vec2::ONE, $vec2::ZERO) });
         }
 
         #[test]
@@ -374,6 +376,9 @@ macro_rules! impl_vec2_tests {
             let mut a = [0 as $t, 0 as $t];
             v.write_to_slice(&mut a);
             assert_eq!(v, $vec2::from_slice(&a));
+
+            should_panic!({ $vec2::ONE.write_to_slice(&mut [0 as $t]) });
+            should_panic!({ $vec2::from_slice(&[0 as $t]) });
         }
 
         #[cfg(feature = "std")]
@@ -494,6 +499,23 @@ macro_rules! impl_vec2_float_tests {
             assert_eq!($new(0.5, 0.25), $new(2.0, 4.0).recip());
         }
 
+        #[test]
+        fn test_project_reject() {
+            assert_eq!($new(0.0, 1.0), $new(1.0, 1.0).project_onto($new(0.0, 2.0)));
+            assert_eq!($new(1.0, 0.0), $new(1.0, 1.0).reject_from($new(0.0, 2.0)));
+            assert_eq!(
+                $new(0.0, 1.0),
+                $new(1.0, 1.0).project_onto_normalized($new(0.0, 1.0))
+            );
+            assert_eq!(
+                $new(1.0, 0.0),
+                $new(1.0, 1.0).reject_from_normalized($new(0.0, 1.0))
+            );
+            should_glam_assert!({ $vec2::ONE.project_onto($vec2::ZERO) });
+            should_glam_assert!({ $vec2::ONE.reject_from($vec2::ZERO) });
+            should_glam_assert!({ $vec2::ONE.project_onto_normalized($vec2::ONE) });
+            should_glam_assert!({ $vec2::ONE.reject_from_normalized($vec2::ONE) });
+        }
         #[test]
         fn test_perp() {
             let v1 = $vec2::new(1.0, 2.0);
@@ -642,6 +664,7 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(0.6, 0.8).clamp_length(10.0, 20.0),
                 $vec2::new(6.0, 8.0) // lengthened to length 10.0
             );
+            should_glam_assert!({ $vec2::ONE.clamp_length(1.0, 0.0) });
         }
 
         #[test]
@@ -671,6 +694,14 @@ macro_rules! impl_vec2_float_tests {
                 $vec2::new(6.0, 8.0) // lengthened to length 10.0
             );
         }
+
+        #[cfg(any(feature = "glam-assert", feature = "debug-glam-assert"))]
+        #[test]
+        fn test_float_glam_assert() {
+            use std::panic::catch_unwind;
+
+            assert!(catch_unwind(|| $vec2::ZERO.normalize()).is_err());
+        }
     };
 }
 
