@@ -150,6 +150,8 @@ macro_rules! impl_vec3_tests {
             assert_eq!(vec(6, 6, 6).clamp(min, max), vec(6, 6, 6));
             assert_eq!(vec(7, 7, 7).clamp(min, max), vec(6, 7, 7));
             assert_eq!(vec(9, 9, 9).clamp(min, max), vec(6, 8, 8));
+
+            should_glam_assert!({ $vec3::clamp($vec3::ZERO, $vec3::ONE, $vec3::ZERO) });
         }
 
         #[test]
@@ -418,6 +420,9 @@ macro_rules! impl_vec3_tests {
             let mut a = [0 as $t, 0 as $t, 0 as $t];
             v.write_to_slice(&mut a);
             assert_eq!(v, $vec3::from_slice(&a));
+
+            should_panic!({ $vec3::ONE.write_to_slice(&mut [0 as $t; 2]) });
+            should_panic!({ $vec3::from_slice(&[0 as $t; 2]) });
         }
 
         #[cfg(feature = "std")]
@@ -551,6 +556,30 @@ macro_rules! impl_vec3_float_tests {
             assert_eq!($new(0.5, 0.25, 0.125), $new(2.0, 4.0, 8.0).recip());
         }
 
+        #[test]
+        fn test_project_reject() {
+            assert_eq!(
+                $new(0.0, 0.0, 1.0),
+                $new(1.0, 0.0, 1.0).project_onto($new(0.0, 0.0, 2.0))
+            );
+            assert_eq!(
+                $new(1.0, 0.0, 0.0),
+                $new(1.0, 0.0, 1.0).reject_from($new(0.0, 0.0, 2.0))
+            );
+            assert_eq!(
+                $new(0.0, 0.0, 1.0),
+                $new(1.0, 0.0, 1.0).project_onto_normalized($new(0.0, 0.0, 1.0))
+            );
+            assert_eq!(
+                $new(1.0, 0.0, 0.0),
+                $new(1.0, 0.0, 1.0).reject_from_normalized($new(0.0, 0.0, 1.0))
+            );
+            should_glam_assert!({ $vec3::ONE.project_onto($vec3::ZERO) });
+            should_glam_assert!({ $vec3::ONE.reject_from($vec3::ZERO) });
+            should_glam_assert!({ $vec3::ONE.project_onto_normalized($vec3::ONE) });
+            should_glam_assert!({ $vec3::ONE.reject_from_normalized($vec3::ONE) });
+        }
+
         #[test]
         fn test_signum() {
             assert_eq!($vec3::ZERO.signum(), $vec3::ONE);
@@ -694,6 +723,7 @@ macro_rules! impl_vec3_float_tests {
                 $vec3::new(0.6, 0.8, 0.0).clamp_length(10.0, 20.0),
                 $vec3::new(6.0, 8.0, 0.0) // lengthened to length 10.0
             );
+            should_glam_assert!({ $vec3::ONE.clamp_length(1.0, 0.0) });
         }
 
         #[test]
@@ -724,8 +754,8 @@ macro_rules! impl_vec3_float_tests {
             );
         }
 
-        #[test]
         #[cfg(feature = "std")]
+        #[test]
         fn test_any_ortho() {
             let eps = 2.0 * core::$t::EPSILON;
 
