@@ -185,6 +185,8 @@ macro_rules! impl_vec4_tests {
             assert_eq!(vec(6, 6, 6, 6).clamp(min, max), vec(6, 6, 6, 6));
             assert_eq!(vec(7, 7, 7, 7).clamp(min, max), vec(6, 6, 7, 7));
             assert_eq!(vec(9, 9, 9, 9).clamp(min, max), vec(6, 6, 8, 8));
+
+            should_glam_assert!({ $vec4::clamp($vec4::ZERO, $vec4::ONE, $vec4::ZERO) });
         }
 
         #[test]
@@ -259,6 +261,9 @@ macro_rules! impl_vec4_tests {
             let mut d = [0 as $t, 0 as $t, 0 as $t, 0 as $t];
             b.write_to_slice(&mut d[..]);
             assert_eq!(a, d);
+
+            should_panic!({ $vec4::ONE.write_to_slice(&mut [0 as $t; 3]) });
+            should_panic!({ $vec4::from_slice(&[0 as $t; 3]) });
         }
 
         // #[test]
@@ -617,6 +622,30 @@ macro_rules! impl_vec4_float_tests {
             );
         }
 
+        #[test]
+        fn test_project_reject() {
+            assert_eq!(
+                $new(0.0, 0.0, 0.0, 1.0),
+                $new(0.0, 1.0, 0.0, 1.0).project_onto($new(0.0, 0.0, 0.0, 2.0))
+            );
+            assert_eq!(
+                $new(0.0, 1.0, 0.0, 0.0),
+                $new(0.0, 1.0, 0.0, 1.0).reject_from($new(0.0, 0.0, 0.0, 2.0))
+            );
+            assert_eq!(
+                $new(0.0, 0.0, 0.0, 1.0),
+                $new(0.0, 1.0, 0.0, 1.0).project_onto_normalized($new(0.0, 0.0, 0.0, 1.0))
+            );
+            assert_eq!(
+                $new(0.0, 1.0, 0.0, 0.0),
+                $new(0.0, 1.0, 0.0, 1.0).reject_from_normalized($new(0.0, 0.0, 0.0, 1.0))
+            );
+            should_glam_assert!({ $vec4::ONE.project_onto($vec4::ZERO) });
+            should_glam_assert!({ $vec4::ONE.reject_from($vec4::ZERO) });
+            should_glam_assert!({ $vec4::ONE.project_onto_normalized($vec4::ONE) });
+            should_glam_assert!({ $vec4::ONE.reject_from_normalized($vec4::ONE) });
+        }
+
         #[test]
         fn test_signum() {
             assert_eq!($vec4::ZERO.signum(), $vec4::ONE);
@@ -668,6 +697,19 @@ macro_rules! impl_vec4_float_tests {
             );
         }
 
+        #[test]
+        fn test_fract() {
+            assert_approx_eq!(
+                $vec4::new(1.35, 1.5, -1.5, 1.999).fract(),
+                $vec4::new(0.35, 0.5, 0.5, 0.999)
+            );
+            assert_approx_eq!(
+                $vec4::new(-0.0, -200000.123, 1000000.123, 1000.9).fract(),
+                $vec4::new(0.0, 0.877, 0.123, 0.9),
+                0.002
+            );
+        }
+
         #[test]
         fn test_ceil() {
             assert_eq!(
@@ -742,6 +784,7 @@ macro_rules! impl_vec4_float_tests {
                 $vec4::new(0.6, 0.8, 0.0, 0.0).clamp_length(10.0, 20.0),
                 $vec4::new(6.0, 8.0, 0.0, 0.0) // lengthened to length 10.0
             );
+            should_glam_assert!({ $vec4::ONE.clamp_length(1.0, 0.0) });
         }
 
         #[test]
